from typing import TYPE_CHECKING


if TYPE_CHECKING:
    import pyspark


def get_spark_session() -> "pyspark.sql.SparkSession":
    import os
    import platform
    from pathlib import Path

    from pyspark.sql import SparkSession

    from layer.jars.spark import additional_jars, layer_jar_path

    if platform.system() == "Windows":
        raise OSError("Running Spark on Windows is not supported yet")

    builder = SparkSession.builder

    spark_jars = [layer_jar_path()]

    # only add additional jars if running outside layer env
    if "LAYER_ENV" not in os.environ:
        spark_jars += additional_jars()

    builder.config("spark.jars", ",".join(spark_jars))

    package_paths = os.environ.get("LAYER_SPARK_PYFILES", "")
    if package_paths:
        builder.config("spark.archives", package_paths)
        builder.config(
            "spark.executorEnv.PYTHONPATH",
            ":".join(Path(p).name for p in package_paths.split(",")),
        )
    return builder.getOrCreate()
