import glob
import hashlib
import inspect
import os
import shutil
import urllib.request
from os import path
from pathlib import Path
from typing import List, Optional

from layer import jars
from layer.config import DEFAULT_LAYER_PATH


_SPARK_ADDITIONAL_JARS_PATH = DEFAULT_LAYER_PATH / "jars"


def layer_jar_path() -> str:
    return path.join(path.dirname(inspect.getfile(jars)), "spark-layer-all.jar")


def additional_jars() -> List[str]:
    _ensure_additional_jars()
    return glob.glob(f"{_SPARK_ADDITIONAL_JARS_PATH}/*.jar")


def _ensure_additional_jars() -> None:
    _ensure_jar_exists(
        "org/apache/hadoop", "hadoop-aws", "3.2.0", "cb3fedee81c0d228dac52e7336cd0e32"
    )
    _ensure_jar_exists(
        "com/amazonaws",
        "aws-java-sdk-bundle",
        "1.11.375",
        "bbee6361a1b26c9ec6a098b1f1d627f0",
    )


def _ensure_jar_exists(group: str, name: str, version: str, checksum: str) -> None:
    if _checksum_jar(name, version) != checksum:
        Path(_SPARK_ADDITIONAL_JARS_PATH).mkdir(parents=True, exist_ok=True)
        repo = f"https://repo1.maven.org/maven2/{group}/{name}/{version}/{name}-{version}.jar"
        with urllib.request.urlopen(repo) as resp, open(
            _jar_file_path(name, version), "wb"
        ) as f:
            shutil.copyfileobj(resp, f)


def _checksum_jar(name: str, version: str) -> Optional[str]:
    jar_file = _jar_file_path(name, version)
    if not os.path.isfile(jar_file):
        return None
    with open(_jar_file_path(name, version), "rb") as f:
        jar_checksum = hashlib.md5()
        while True:
            chunk = f.read(8192)
            if not chunk:
                break
            jar_checksum.update(chunk)
        return jar_checksum.hexdigest()


def _jar_file_path(name: str, version: str) -> Path:
    return _SPARK_ADDITIONAL_JARS_PATH / f"{name}-{version}.jar"
