from pathlib import Path
from typing import Any, Dict, Optional

from layer.projects.asset import AssetPath, AssetType

from ..client import DatasourceRef, DatasourceType, RawDataset
from .common import FieldChain, ProjectDefinitionParser


class RawDatasetDefinitionParser(ProjectDefinitionParser):
    def __init__(
        self,
        payload: Dict[str, Any],
        path: Path,
        project_name: str,
        field_chain: Optional[FieldChain] = None,
    ):
        super().__init__(path, project_name, "raw_dataset.json", payload, field_chain)

    def parse(self) -> RawDataset:
        _data = self._safe_payload()
        integration = _data.get("integration", {})
        integration_name = integration.get("name", None)
        table_name = integration.get("table", None)
        name = _data.get("name", None)
        description = _data.get("description", "")

        asset_path = AssetPath(
            asset_type=AssetType.DATASET,
            entity_name=name,
            project_name=self._project_name,
        )

        return RawDataset(
            asset_path=asset_path,
            description=description,
            datasource=DatasourceRef(name=integration_name, type=DatasourceType.TABLE),
        ).with_table_name(table_name)
